@@ -584,6 +584,9 @@ private:
 
   // Ternary expressions.
   llvm::Value *EmitReg_CondExpr(tree_node *op0, tree_node *op1, tree_node *op2);
+#if (GCC_MINOR > 5)
+  llvm::Value *EmitReg_FMA_EXPR(tree_node *op0, tree_node *op1, tree_node *op2);
+#endif
 #if (GCC_MINOR > 6)
   llvm::Value *EmitReg_VEC_PERM_EXPR(tree_node *op0, tree_node *op1,
                                      tree_node *op2);
@@ -634,6 +637,7 @@ private:
   llvm::Value *EmitBuiltinLFLOOR(gimple_statement_d *stmt);
   llvm::Value *EmitBuiltinLROUND(gimple_statement_d *stmt);
   llvm::Value *EmitBuiltinCEXPI(gimple_statement_d *stmt);
+  llvm::Value *EmitBuiltinSIGNBIT(gimple_statement_d *stmt);
 
   bool EmitBuiltinAdjustTrampoline(gimple_statement_d *stmt,
                                    llvm::Value *&Result);
@@ -651,8 +655,7 @@ private:
                                     llvm::Value *&Result);
   bool EmitBuiltinFrobReturnAddr(gimple_statement_d *stmt,
                                  llvm::Value *&Result);
-  bool EmitBuiltinInitTrampoline(gimple_statement_d *stmt,
-                                 llvm::Value *&Result);
+  bool EmitBuiltinInitTrampoline(gimple_statement_d *stmt, bool OnStack);
   bool EmitBuiltinMemCopy(gimple_statement_d *stmt, llvm::Value *&Result,
                           bool isMemMove, bool SizeCheck);
   bool EmitBuiltinMemSet(gimple_statement_d *stmt, llvm::Value *&Result,
