@@ -323,9 +323,9 @@ static bool SizeOfGlobalMatchesDecl(GlobalValue *GV, tree decl) {
 #error LLVM_TARGET_NAME macro not specified
 #endif
 
-/// ConfigureLLVM - Initialized and configure LLVM.
+/// ConfigureLLVM - Initialize and configure LLVM.
 static void ConfigureLLVM(void) {
-// Initialize the LLVM backend.
+  // Initialize the LLVM backend.
 #define DoInit2(TARG, MOD) LLVMInitialize##TARG##MOD()
 #define DoInit(T, M) DoInit2(T, M)
   DoInit(LLVM_TARGET_NAME, TargetInfo);
@@ -479,42 +479,55 @@ static void CreateTargetMachine(const std::string &TargetTriple) {
 #endif
 
   TargetOptions Options;
+
+  // Set frame pointer elimination mode.
   if (flag_omit_frame_pointer) {
     // Eliminate frame pointers everywhere.
     Options.NoFramePointerElim = false;
-    Options.NoFramePointerElimNonLeaf = false;
   } else {
     // Keep frame pointers everywhere.
     Options.NoFramePointerElim = true;
-    Options.NoFramePointerElimNonLeaf = true;
   }
   // If a target has an option to eliminate frame pointers in leaf functions
   // only then it should set
   //   NoFramePointerElim = false;
   //   NoFramePointerElimNonLeaf = true;
   // in its LLVM_SET_TARGET_MACHINE_OPTIONS method when this option is true.
+
+#ifdef HAVE_INITFINI_ARRAY
+  Options.UseInitArray = true;
+#else
+  Options.UseInitArray = false;
+#endif
+
+  // TODO: Set float ABI type.
+
+  // TODO: Set FP fusion mode.
+
+  // TODO: LessPreciseFPMADOption.
+  Options.NoInfsFPMath = flag_finite_math_only;
+  Options.NoNaNsFPMath = flag_finite_math_only;
+  Options.NoZerosInBSS = !flag_zero_initialized_in_bss;
   Options.UnsafeFPMath =
 #if (GCC_MINOR > 5)
       fast_math_flags_set_p(&global_options);
 #else
   fast_math_flags_set_p();
 #endif
-  Options.NoInfsFPMath = flag_finite_math_only;
-  Options.NoNaNsFPMath = flag_finite_math_only;
-  Options.NoZerosInBSS = !flag_zero_initialized_in_bss;
+  // TODO: UseSoftFloat.
+  // TODO: StackAlignmentOverride.
+  // TODO: RealignStack.
+  // TODO: DisableTailCalls.
+  // TODO: TrapFuncName.
   Options.PositionIndependentExecutable = flag_pie;
 #if (GCC_MINOR > 5)
   Options.EnableSegmentedStacks = flag_split_stack;
 #endif
-#ifdef HAVE_INITFINI_ARRAY
-  Options.UseInitArray = true;
-#else
-  Options.UseInitArray = false;
-#endif
-  Options.SSPBufferSize = PARAM_VALUE(PARAM_SSP_BUFFER_SIZE);
+
 #ifdef LLVM_SET_TARGET_MACHINE_OPTIONS
   LLVM_SET_TARGET_MACHINE_OPTIONS(Options);
 #endif
+
   TheTarget = TME->createTargetMachine(TargetTriple, CPU, FeatureStr, Options,
                                        RelocModel, CMModel, CodeGenOptLevel());
   assert(TheTarget->getDataLayout()->isBigEndian() == BYTES_BIG_ENDIAN);
@@ -562,6 +575,8 @@ static void CreateModule(const std::string &TargetTriple) {
     IdentString += version_string;
     output_ident(IdentString.c_str());
   }
+#else
+  (void)output_ident; // Avoid compiler warning about output_ident being unused.
 #endif
 #endif
 
@@ -642,7 +657,6 @@ static void InitializeBackend(void) {
 
   // Configure the pass builder.
   PassBuilder.SizeLevel = optimize_size;
-  PassBuilder.DisableSimplifyLibCalls = flag_no_simplify_libcalls;
   PassBuilder.DisableUnitAtATime = !flag_unit_at_a_time;
   PassBuilder.DisableUnrollLoops = !flag_unroll_loops;
 //  Don't turn on the SLP vectorizer by default at -O3 for the moment.
@@ -663,7 +677,7 @@ static void InitializeOutputStreams(bool Binary) {
   std::string Error;
 
   OutStream = new raw_fd_ostream(llvm_asm_file_name, Error,
-                                 Binary ? raw_fd_ostream::F_Binary : 0);
+                                 Binary ? sys::fs::F_Binary : sys::fs::F_None);
 
   if (!Error.empty())
     report_fatal_error(Error);
@@ -680,6 +694,7 @@ static void createPerFunctionOptimizationPasses() {
   // FIXME: Move the code generator to be function-at-a-time.
   PerFunctionPasses = new FunctionPassManager(TheModule);
   PerFunctionPasses->add(new DataLayout(TheModule));
+  TheTarget->addAnalysisPasses(*PerFunctionPasses);
 
 #ifndef NDEBUG
   PerFunctionPasses->add(createVerifierPass());
@@ -727,9 +742,11 @@ static void createPerModuleOptimizationPasses() {
   PerModulePasses->add(new DataLayout(TheModule));
   TheTarget->addAnalysisPasses(*PerModulePasses);
 
-  bool NeedAlwaysInliner = false;
-  llvm::Pass *InliningPass = 0;
-  if (flag_inline_small_functions && !flag_no_inline) {
+  Pass *InliningPass;
+  if (!LLVMIROptimizeArg)
+    // If the user asked for no LLVM optimization, then don't do any inlining.
+    InliningPass = 0;
+  else if (flag_inline_small_functions && !flag_no_inline) {
     // Inline small functions.  Figure out a reasonable threshold to pass llvm's
     // inliner.  GCC has many options that control inlining, but we have decided
     // not to support anything like that for dragonegg.
@@ -743,19 +760,9 @@ static void createPerModuleOptimizationPasses() {
       Threshold = 225;
     InliningPass = createFunctionInliningPass(Threshold);
   } else {
-    // If full inliner is not run, check if always-inline is needed to handle
-    // functions that are  marked as always_inline.
+    // Run the always-inline pass to handle functions marked as always_inline.
     // TODO: Consider letting the GCC inliner do this.
-    for (Module::iterator I = TheModule->begin(), E = TheModule->end(); I != E;
-         ++I)
-      if (I->getAttributes().hasAttribute(AttributeSet::FunctionIndex,
-                                          Attribute::AlwaysInline)) {
-        NeedAlwaysInliner = true;
-        break;
-      }
-
-    if (NeedAlwaysInliner)
-      InliningPass = createAlwaysInlinerPass(); // Inline always_inline funcs
+    InliningPass = createAlwaysInlinerPass();
   }
 
   PassBuilder.OptLevel = ModuleOptLevel();
@@ -945,10 +952,11 @@ static void emit_alias(tree decl, tree target) {
   }
 
   GlobalValue *Aliasee = 0;
+  bool IsWeakRef = lookup_attribute("weakref", DECL_ATTRIBUTES(decl));
   if (isa<IDENTIFIER_NODE>(target)) {
     StringRef AliaseeName(IDENTIFIER_POINTER(target),
                           IDENTIFIER_LENGTH(target));
-    if (!lookup_attribute("weakref", DECL_ATTRIBUTES(decl))) {
+    if (!IsWeakRef) {
       Aliasee = TheModule->getNamedValue(AliaseeName);
       if (!Aliasee)
         Aliasee = TheModule->getNamedValue(("\1" + AliaseeName).str());
@@ -976,7 +984,7 @@ static void emit_alias(tree decl, tree target) {
 
   GlobalValue::LinkageTypes Linkage = GetLinkageForAlias(decl);
 
-  if (Linkage != GlobalValue::InternalLinkage) {
+  if (Linkage != GlobalValue::InternalLinkage && !IsWeakRef) {
     // Create the LLVM alias.
     GlobalAlias *GA =
         new GlobalAlias(Aliasee->getType(), Linkage, "", Aliasee, TheModule);
