@@ -231,7 +231,7 @@ StringRef DebugInfo::getFunctionName(tree FnDecl) {
 
 /// EmitFunctionStart - Constructs the debug code for entering a function.
 void DebugInfo::EmitFunctionStart(tree FnDecl, Function *Fn) {
-  DIType FNType = getOrCreateType(TYPE_MAIN_VARIANT(TREE_TYPE(FnDecl)));
+  DIType FNType = getOrCreateType(TREE_TYPE(FnDecl));
 
   unsigned lineno = CurLineNo;
 
@@ -411,6 +411,8 @@ void DebugInfo::EmitGlobalVariable(GlobalVariable *GV, tree decl) {
   expanded_location Loc = expand_location(DECL_SOURCE_LOCATION(decl));
   DIType TyD = getOrCreateType(TREE_TYPE(decl));
   StringRef DispName = GV->getName();
+  if (DispName.empty())
+    DispName = "__unknown__";
   if (DECL_NAME(decl)) {
     if (IDENTIFIER_POINTER(DECL_NAME(decl)))
       DispName = IDENTIFIER_POINTER(DECL_NAME(decl));
@@ -918,7 +920,6 @@ DIType DebugInfo::getOrCreateType(tree type) {
   DIType Ty;
   switch (TREE_CODE(type)) {
   case ERROR_MARK:
-  case LANG_TYPE:
   case TRANSLATION_UNIT_DECL:
   default:
     llvm_unreachable("Unsupported type");
@@ -926,6 +927,13 @@ DIType DebugInfo::getOrCreateType(tree type) {
 #if (GCC_MINOR > 5)
   case NULLPTR_TYPE:
 #endif
+  case LANG_TYPE: {
+    tree name = TYPE_NAME(type);
+    if (TREE_CODE(name) == TYPE_DECL)
+      name = DECL_NAME(name);
+    return Builder.createUnspecifiedType(IDENTIFIER_POINTER(name));
+  }
+
   case OFFSET_TYPE:
   case POINTER_TYPE:
   case REFERENCE_TYPE:
@@ -968,6 +976,10 @@ DIType DebugInfo::getOrCreateType(tree type) {
 /// initialization is done.
 void DebugInfo::Initialize() {
 
+  // Debug info metadata without a version or with an outdated version will be
+  // dropped. Add a version here to avoid that.
+  M.addModuleFlag(llvm::Module::Error, "Debug Info Version",
+                  llvm::DEBUG_METADATA_VERSION);
   // Each input file is encoded as a separate compile unit in LLVM
   // debugging information output. However, many target specific tool chains
   // prefer to encode only one compile unit in an object file. In this
@@ -1091,7 +1103,7 @@ DICompositeType DebugInfo::CreateCompositeType(
   case dwarf::DW_TAG_structure_type:
     return Builder.createStructType(Context, Name, F, LineNumber, SizeInBits,
                                     AlignInBits, Flags, DerivedFrom, Elements,
-                                    0, ContainingType);
+                                    0, DIType(ContainingType));
   case dwarf::DW_TAG_union_type:
     return Builder.createUnionType(Context, Name, F, LineNumber, SizeInBits,
                                    AlignInBits, Flags, Elements, RuntimeLang);
@@ -1115,11 +1127,14 @@ DISubprogram DebugInfo::CreateSubprogram(
     StringRef LinkageName, DIFile F, unsigned LineNo, DIType Ty,
     bool isLocalToUnit, bool isDefinition, unsigned VK, unsigned VIndex,
     DIType ContainingType, unsigned Flags, bool isOptimized, Function *Fn) {
+  DICompositeType CTy = getDICompositeType(Ty);
+  assert(CTy.Verify() && "Expected a composite type!");
   if (ContainingType.isValid() || VK || VIndex)
-    return Builder.createMethod(Context, Name, LinkageName, F, LineNo, Ty,
-                                isLocalToUnit, isDefinition, VK, VIndex, NULL,
+    return Builder.createMethod(Context, Name, LinkageName, F, LineNo, CTy,
+                                isLocalToUnit, isDefinition, VK, VIndex,
+                                DIType(),
                                 Flags, isOptimized, Fn, NULL);
-  return Builder.createFunction(Context, Name, LinkageName, F, LineNo, Ty,
+  return Builder.createFunction(Context, Name, LinkageName, F, LineNo, CTy,
                                 isLocalToUnit, isDefinition, LineNo, Flags,
                                 isOptimized, Fn, NULL, NULL);
 }
